/*
 * File: grc_defs.h
 *
 * File derived from Microchip Software Solutions Library "Graphics/Primitive.h".
 *
 * Contains definitions required to use bitmap graphics images from files generated
 * with Microchip 'Graphics Resources Converter' (GRC).
 *
 * Dependencies:  Microchip "GenericTypeDefs.h"
 */
#ifndef _GRAPHICS_H
#define _GRAPHICS_H

#include "Common/GenericTypeDefs.h"

#define GRAPHICS_LIBRARY_VERSION  (0x0306)

// Size of header data block of bitmap image definition for 1-bit-per-pixel image...
// The actual image data (byte array) follows the header block.
#define GRC_IMAGE_HEADER_SIZE_1BPP   10   // bytes

// Flash data with 32bit pointers
#define FLASH_BYTE  const BYTE
#define FLASH_WORD  const WORD
#define FLASH_DWORD const DWORD


/*********************************************************************
* Overview: Memory type enumeration to determine the source of data.
*           Used in interpreting bitmap and font from different
*           memory sources.
*
*********************************************************************/
typedef enum
{
    FLASH         = 0x0000,                 // internal flash
    EXTERNAL      = 0x0001,                 // external memory
    FLASH_JPEG    = 0x0002,                 // internal flash
    EXTERNAL_JPEG = 0x0003,                 // external memory
    RAM           = 0x0004,                 // RAM
    EDS_EPMP      = 0x0005,                 // memory in EPMP, base addresses are
                                            // are set in the hardware profile

    IMAGE_MBITMAP = 0x0000,                 // data resource is type Microchip bitmap
    IMAGE_JPEG    = 0x0100,                 // data resource is type JPEG

    COMP_NONE     = 0x0000,                 // no compression
    COMP_RLE      = 0x1000,                 // compressed with RLE
    COMP_IPU      = 0x2000,                 // compressed with DEFLATE (for IPU)
} GFX_RESOURCE;


#define GFX_COMP_MASK   0xF000
#define GFX_MEM_MASK    0x000F

/*********************************************************************
* Overview: Structure describing the bitmap header.
*
*********************************************************************/
typedef struct
{
    BYTE        compression;                // Compression setting
    BYTE        colorDepth;                 // Color depth used
    SHORT       height;                     // Image height
    SHORT       width;                      // Image width
} BITMAP_HEADER;

/*********************************************************************
* Overview: Structure describing the partial image area to render.
*
*********************************************************************/
typedef struct
{
    WORD         width;                       // Parital Image width
    WORD         height;                      // Partial Image height
    SHORT        xoffset;                     // xoffset of image
    SHORT        yoffset;                     // yoffset of image
} PUTIMAGE_PARAM;

/*********************************************************************
* Overview: Structure for images stored in FLASH memory.
*
*********************************************************************/
typedef struct
{
    GFX_RESOURCE type;                      // must be FLASH
    FLASH_BYTE   *address;                  // image address in FLASH
} IMAGE_FLASH;


/*********************************************************************
* Overview: Structure for images stored in RAM memory.
*
*********************************************************************/
typedef struct
{
    GFX_RESOURCE type;                      // must be RAM
    DWORD        *address;                  // image address in RAM
} IMAGE_RAM;


#endif  // _GRAPHICS_H
